@@ -1,11 +1,12 @@
 """
 Functions to run extraction wrangles
 """
-import pandas as _pd
 from typing import Union as _Union
+import re as _re
+from collections import OrderedDict as _OrderedDict
+import pandas as _pd
 from .. import extract as _extract
 from .. import format as _format
-import re as _re
 
 
 def address(df: _pd.DataFrame, input: str, output: str, dataType: str) -> _pd.DataFrame:
@@ -27,20 +28,29 @@ def address(df: _pd.DataFrame, input: str, output: str, dataType: str) -> _pd.Da
           - string
           - array
         description: Name of the output column.
+      dataType:
+        type: string
+        description: Specific part of the address to extract
+        enum:
+          - streets
+          - cities
+          - regions
+          - countries
     """
-    # If the input is a string
-    if isinstance(input, str) and isinstance(output, str):
-        df[output] = _extract.address(df[input].astype(str).tolist(), dataType)
-    
-    # If the input is multiple columns (a list)
-    elif isinstance(input, list) and isinstance(output, list):
-        for in_col, out_col in zip(input, output):
-            df[out_col] = _extract.address(df[in_col].astype(str).tolist(), dataType)
-            
-    # If the input and output are not the same type    
-    elif type(input) != type(output):
-        raise ValueError('If providing a list of inputs/outputs, a corresponding list of inputs/outputs must also be provided.')
-            
+    # If output is not specified, overwrite input columns in place
+    if output is None: output = input
+
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+  
+    for input_column, output_column in zip(input, output):
+        df[output_column] = _extract.address(df[input_column].astype(str).tolist(), dataType)
+  
     return df
 
 
@@ -104,200 +114,27 @@ def attributes(df: _pd.DataFrame, input: str, output: str, responseContent: str
           - mid
           - max
     """
-    # If the input is a string
-    if isinstance(input, str) and isinstance(output, str):
-        df[output] = _extract.attributes(df[input].astype(str).tolist(), responseContent, attribute_type, desired_unit, bound)
-        
-    # If the input is multiple columns (a list)
-    elif isinstance(input, list) and isinstance(output, list):
-        for in_col, out_col in zip(input, output):
-            df[out_col] = _extract.attributes(df[in_col].astype(str).tolist(), responseContent, attribute_type, desired_unit, bound)
-    
-    # If the input and output are not the same type    
-    elif type(input) != type(output):
-        raise ValueError('If providing a list of inputs/outputs, a corresponding list of inputs/outputs must also be provided.')
-        
-    return df
-
-
-def codes(df: _pd.DataFrame, input: _Union[str, list], output: _Union[str, list]) -> _pd.DataFrame:
-    """
-    type: object
-    description: Extract alphanumeric codes from the input. Requires WrangleWorks Account.
-    additionalProperties: false
-    required:
-      - input
-      - output
-    properties:
-      input:
-        type:
-          - string
-          - array
-        description: Name or list of input columns.
-      output:
-        type:
-          - string
-          - array
-        description: Name or list of output columns
-    """
-    if isinstance(input, str):
-        df[output] = _extract.codes(df[input].astype(str).tolist())
-    elif isinstance(input, list):
-        # If a list of inputs is provided, ensure the list of outputs is the same length
-        if len(input) != len(output):
-            if isinstance(output, str):
-                df[output] = _extract.codes(_format.concatenate(df[input].astype(str).values.tolist(), ' aaaaaaa '))
-            else:
-                raise ValueError('If providing a list of inputs, a corresponding list of outputs must also be provided.')
-        else:
-            for input_column, output_column in zip(input, output):
-                df[output_column] = _extract.codes(df[input_column].astype(str).tolist())
-
-    return df
-
-
-def custom(df: _pd.DataFrame, input: _Union[str, list], output: _Union[str, list], model_id: _Union[str, list] = None, find: str = None) -> _pd.DataFrame:
-    """
-    type: object
-    description: Extract data from the input using a DIY or bespoke extraction wrangle. Requires WrangleWorks Account and Subscription.
-    additionalProperties: false
-    required:
-      - input
-      - output
-    properties:
-      input:
-        type:
-          - string
-          - array
-        description: Name or list of input columns.
-      output:
-        type:
-          - string
-          - array
-        description: Name or list of output columns
-      model_id:
-        type: string
-        description: The ID of the wrangle to use (do not include find)
-      find:
-        type:
-          - string
-        description: Pattern to find using regex (do not include model_id)
-    """
-    if model_id is not None and find is None:
-    
-        if isinstance(input, str) and isinstance(output, str) and type(model_id) != list:
-            df[output] = _extract.custom(df[input].astype(str).tolist(), model_id=model_id)
-            
-        elif isinstance(input, list) and type(model_id) != list:
-            # If a list of inputs is provided, ensure the list of outputs is the same length
-            if len(input) != len(output):
-                if isinstance(output, str):
-                    df[output] = _extract.custom(_format.concatenate(df[input].astype(str).values.tolist(), ' '), model_id=model_id)
-                else:
-                    raise ValueError('If providing a list of inputs, a corresponding list of outputs must also be provided.')
-            else:
-                for input_column, output_column in zip(input, output):
-                    df[output_column] = _extract.custom(df[input_column].astype(str).tolist(), model_id=model_id)
-                    
-        # Multiple different custom wrangles at the same time
-        elif isinstance(input, list) and isinstance(output, list) and isinstance(model_id, list):
-            for in_col, out_col, mod_id in zip(input, output, model_id):
-                df[out_col] = _extract.custom(df[in_col].astype(str).tolist(),model_id=mod_id)
-
-    elif find is not None and model_id is None:
-        
-        def mini_extract(string):
-            new_string = _re.findall(find, string)
-            return new_string
-        
-        df[output] = df[input].apply(lambda x: mini_extract(x))
-        
-    else:
-        raise ValueError('Extract custom must have model_id or find as parameters')
-          
-    return df
-
-
-def html(df: _pd.DataFrame, input: _Union[str, list], data_type: str, output: _Union[str, list] = None) -> _pd.DataFrame:
-    """
-    type: object
-    description: Extract elements from strings containing html. Requires WrangleWorks Account.
-    additionalProperties: false
-    required:
-      - input
-      - output
-      - data_type
-    properties:
-      input:
-        type:
-          - string
-          - array
-        description: Name or list of input columns.
-      output:
-        type:
-          - string
-          - array
-        description: Name or list of output columns
-      data_type:
-        type: string
-        description: The type of data to extract
-        enum:
-          - text
-          - links
-    """
-    if isinstance(input, str): input = [input]
+    # If output is not specified, overwrite input columns in place
     if output is None: output = input
 
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+
+    # Loop through and apply for all columns
     for input_column, output_column in zip(input, output):
-        df[output_column] = _extract.html(df[input_column].astype(str).tolist(), dataType=data_type)
-            
-    return df
-
-
-def properties(df: _pd.DataFrame, input: str, output: str, property_type: str = None, return_data_type: str = 'list') -> _pd.DataFrame:
-    """
-    type: object
-    description: Extract text properties from the input. Requires WrangleWorks Account.
-    additionalProperties: false
-    required:
-      - input
-      - output
-    properties:
-      input:
-        type:
-          - string
-          - array
-        description: Name of the input column
-      output:
-        type:
-          - string
-          - array
-        description: Name of the output columns
-      property_type:
-        type: string
-        description: The specific type of properties to extract
-      return_data_type:
-        type: string
-        description: The format to return the data, as a list or as a string
-        enum:
-          - Colours
-          - Materials
-          - Shapes
-          - Standards
-    """
-    # If the input is a string
-    if isinstance(input, str) and isinstance(output, str):
-        df[output] = _extract.properties(df[input].astype(str).tolist(), type=property_type, return_data_type=return_data_type)
-    
-    # If the input is multiple columns (a list)
-    elif isinstance(input, list) and isinstance(output, list):
-        for in_col, out_col in zip(input, output):
-            df[out_col] = _extract.properties(df[in_col].astype(str).tolist(), type=property_type, return_data_type=return_data_type)
-    
-    # If the input and output are not the same type
-    elif type(input) != type(output):
-        raise ValueError('If providing a list of inputs/outputs, a corresponding list of inputs/outputs must also be provided.')
-    
+        df[output_column] = _extract.attributes(
+            df[input_column].astype(str).tolist(),
+            responseContent,
+            attribute_type,
+            desired_unit,
+            bound
+        )
+        
     return df
 
 
@@ -321,19 +158,145 @@ def brackets(df: _pd.DataFrame, input: str, output: str) -> _pd.DataFrame:
           - array
         description: Name of the output columns
     """
-    # If the input is a string
-    if isinstance(input, str) and isinstance(output, str):
-        df[output] = _extract.brackets(df[input].astype(str).tolist())
+    # If output is not specified, overwrite input columns in place
+    if output is None: output = input
 
-    # If the input is multiple columns (a list)
-    elif isinstance(input, list) and isinstance(output, list):
-        for in_col, out_col in zip(input, output):
-            df[out_col] = _extract.brackets(df[in_col].astype(str).tolist())
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+
+    # Loop through and apply for all columns
+    for input_column, output_column in zip(input, output):
+        df[output_column] = _extract.brackets(df[input_column].astype(str).tolist())
+
+    return df
+
+
+def codes(df: _pd.DataFrame, input: _Union[str, list], output: _Union[str, list]) -> _pd.DataFrame:
+    """
+    type: object
+    description: Extract alphanumeric codes from the input. Requires WrangleWorks Account.
+    additionalProperties: false
+    required:
+      - input
+      - output
+    properties:
+      input:
+        type:
+          - string
+          - array
+        description: Name or list of input columns.
+      output:
+        type:
+          - string
+          - array
+        description: Name or list of output columns
+    """
+    # If output is not specified, overwrite input columns in place
+    if output is None: output = input
+
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    if len(output) == 1 and len(input) > 1:
+        df[output[0]] = _extract.codes(df[input].astype(str).aggregate(' AAA '.join, axis=1).tolist())
+    else:
+        # Ensure input and output are equal lengths
+        if len(input) != len(output) and len(output) > 1:
+            raise ValueError('The lists for input and output must be the same length.')
+
+        # Loop through and apply for all columns
+        for input_column, output_column in zip(input, output):
+            df[output_column] = _extract.codes(df[input_column].astype(str).tolist())
+
+    return df
+
+
+def custom(df: _pd.DataFrame, input: list, output: _Union[str, list], model_id: _Union[str, list], use_labels: bool = False) -> _pd.DataFrame:
+    """
+    type: object
+    description: Extract data from the input using a DIY or bespoke extraction wrangle. Requires WrangleWorks Account and Subscription.
+    additionalProperties: true
+    required:
+      - input
+      - output
+      - model_id
+    properties:
+      input:
+        type:
+          - string
+          - array
+        description: Name or list of input columns.
+      output:
+        type:
+          - string
+          - array
+        description: Name or list of output columns
+      model_id:
+        type: string
+        description: The ID of the wrangle to use
+    """
+    if not output:
+        output = input
+
+    if not isinstance(output, list):
+        output = [output]
+
+    if not isinstance(model_id, list):
+        # If a list of inputs is provided, ensure the list of outputs is the same length
+        if len(input) != len(output):
+            if len(output) == 1:
+                df[output[0]] = _extract.custom(_format.concatenate(df[input].astype(str).values.tolist(), ' '), model_id=model_id)
+            else:
+                raise ValueError('If providing a list of inputs, a corresponding list of outputs must also be provided.')
+        else:
+            for input_column, output_column in zip(input, output):
+                df[output_column] = _extract.custom(df[input_column].astype(str).tolist(), model_id=model_id)
+                
+    # Multiple different custom wrangles at the same time
+    else:
+        for in_col, out_col, mod_id in zip(input, output, model_id):
+            df[out_col] = _extract.custom(df[in_col].astype(str).tolist(),model_id=mod_id)
     
-    # If the input and output are not the same type
-    elif type(input) != type(output):
-        raise ValueError('If providing a list of inputs/outputs, a corresponding list of inputs/outputs must also be provided.')
+    if use_labels:
+        if len(output) > 1:
+            raise ValueError("'use_labels' can only be used with a single output")
+        
+        output = output[0]
 
+        # Run the custom dictionary maker after normal operation from extract
+        # This will be triggered only if a parameter is set
+        result = []
+        for out_row in df[output]:
+        
+            dict_output = {'Unlabeled': []}
+            # Iterating over the results
+            for item in out_row:
+                
+                try:
+                    item = item.strip()
+                    # Check if the item contains a colon
+                    if (item.count(':') == 1 and item.split(':')[0] != ''):
+                        dict_output[item.split(':')[0].strip()] = item.split(':')[1].strip()
+                    else:
+                        dict_output['Unlabeled'].append(item)
+                except:
+                    dict_output['Unlabeled'].append(item)
+                    
+            tmp_unlabeled = dict_output['Unlabeled']
+            del dict_output['Unlabeled']
+            output_dict = _OrderedDict(dict_output)
+            output_dict['Unlabeled'] = tmp_unlabeled
+                
+            result.append(dict(output_dict))
+            
+        df[output] = result
+        
     return df
 
 
@@ -347,10 +310,14 @@ def date_properties(df: _pd.DataFrame, input: _pd.Timestamp, property: str, outp
       - property
     properties:
       input:
-        type: string
+        type:
+          - string
+          - array
         description: Name of the input column
       output:
-        type: string
+        type:
+          - string
+          - array
         description: Name of the output columns
       property:
         type: string
@@ -365,26 +332,37 @@ def date_properties(df: _pd.DataFrame, input: _pd.Timestamp, property: str, outp
           - week_year
           - quarter
     """
+    # If output is not specified, overwrite input columns in place
     if output is None: output = input
-    
-    # Converting data to datetime
-    df[input] = _pd.to_datetime(df[input])
-    
-    properties_object = {
-        'day': df[input].dt.day,
-        'day_of_year': df[input].dt.day_of_year,
-        'month': df[input].dt.month,
-        'month_name': df[input].dt.month_name(),
-        'weekday': df[input].dt.weekday,
-        'week_day_name': df[input].dt.day_name(),
-        'week_year': df[input].dt.isocalendar()['week'],
-        'quarter': df[input].dt.quarter,
-    }
-    
-    if property in properties_object.keys():
-        df[output] = properties_object[property]
-    else:
-        raise ValueError(f"\"{property}\" not a valid date property.")
+
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+
+    # Loop through and apply for all columns
+    for input_column, output_column in zip(input, output):
+        # Converting data to datetime
+        df_temp = _pd.to_datetime(df[input_column])
+        
+        properties_object = {
+            'day': df_temp.dt.day,
+            'day_of_year': df_temp.dt.day_of_year,
+            'month': df_temp.dt.month,
+            'month_name': df_temp.dt.month_name(),
+            'weekday': df_temp.dt.weekday,
+            'week_day_name': df_temp.dt.day_name(),
+            'week_year': df_temp.dt.isocalendar()['week'],
+            'quarter': df_temp.dt.quarter,
+        }
+        
+        if property in properties_object.keys():
+            df[output_column] = properties_object[property]
+        else:
+            raise ValueError(f"\"{property}\" not a valid date property.")
     
     return df
 
@@ -470,3 +448,134 @@ def date_range(df: _pd.DataFrame, start_time: _pd.Timestamp, end_time: _pd.Times
     df[output] = results
     
     return df
+
+
+def html(df: _pd.DataFrame, input: _Union[str, list], data_type: str, output: _Union[str, list] = None) -> _pd.DataFrame:
+    """
+    type: object
+    description: Extract elements from strings containing html. Requires WrangleWorks Account.
+    additionalProperties: false
+    required:
+      - input
+      - output
+      - data_type
+    properties:
+      input:
+        type:
+          - string
+          - array
+        description: Name or list of input columns.
+      output:
+        type:
+          - string
+          - array
+        description: Name or list of output columns
+      data_type:
+        type: string
+        description: The type of data to extract
+        enum:
+          - text
+          - links
+    """
+    # If output is not specified, overwrite input columns in place
+    if output is None: output = input
+
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+    
+    # Loop through and apply for all columns
+    for input_column, output_column in zip(input, output):
+        df[output_column] = _extract.html(df[input_column].astype(str).tolist(), dataType=data_type)
+            
+    return df
+
+
+def properties(df: _pd.DataFrame, input: str, output: str, property_type: str = None, return_data_type: str = 'list') -> _pd.DataFrame:
+    """
+    type: object
+    description: Extract text properties from the input. Requires WrangleWorks Account.
+    additionalProperties: false
+    required:
+      - input
+      - output
+    properties:
+      input:
+        type:
+          - string
+          - array
+        description: Name of the input column
+      output:
+        type:
+          - string
+          - array
+        description: Name of the output columns
+      property_type:
+        type: string
+        description: The specific type of properties to extract
+      return_data_type:
+        type: string
+        description: The format to return the data, as a list or as a string
+        enum:
+          - Colours
+          - Materials
+          - Shapes
+          - Standards
+    """
+    # If output is not specified, overwrite input columns in place
+    if output is None: output = input
+
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+    
+    # Loop through and apply for all columns
+    for input_column, output_column in zip(input, output):
+        df[output_column] = _extract.properties(df[input_column].astype(str).tolist(), type=property_type, return_data_type=return_data_type)
+    
+    return df
+
+
+def regex(df: _pd.DataFrame, input: str, find: str, output: str) -> _pd.DataFrame:
+    """
+    type: object
+    description: Extract single values using regex
+    additionalProperties: false
+    required:
+      - input
+      - output
+    properties:
+      input:
+        type: string
+        description: Name of the input column.
+    output:
+      type: string
+      description: Name of the output column.
+      find:
+        type: string
+        description: Pattern to find using regex
+    """
+    # If output is not specified, overwrite input columns in place
+    if output is None: output = input
+
+    # If a string provided, convert to list
+    if not isinstance(input, list): input = [input]
+    if not isinstance(output, list): output = [output]
+
+    # Ensure input and output are equal lengths
+    if len(input) != len(output):
+        raise ValueError('The lists for input and output must be the same length.')
+    
+    # Loop through and apply for all columns
+    for input_column, output_column in zip(input, output):
+        df[output_column] = df[input_column].apply(lambda x: _re.findall(find, x))
+    
+    return df
